


/*
* @version  0.1.1
* @author   Lauri Rooden - https://github.com/litejs/fn-lite
* @license  MIT License  - http://lauri.rooden.ee/mit-license.txt
*/


function Nop()   {}
function True()  { return true  }
function False() { return false }
function This()  { return this  }
function Init()  {
	var self = this
	return self.init && self.init.apply(self, arguments) || self
}

!function(root) {
	var P = "prototype"
	, A = Array[P], D = Date[P], F = Function[P], N = Number[P], S = String[P]
	, O = Object
	, sl = F.call.bind(A.slice)
	, cs = []




	// Function extensions
	// -------------------


	F.construct = function(a) {
		/*
		* bind version have bad performance and memory consumption
		* return new(F.bind.apply(this, A.concat.apply([null], a)))
		*/
		var l = a.length
		return l ? (cs[l] || (cs[l] = Fn("t a->new t(a["+Object.keys(sl(a)).join("],a[")+"])")))(this, a) : new this
	}

	F.partial = function() {
		var self = this, a = sl(arguments)
		return function() {return self.apply(this, A.concat.apply(a, arguments))}
	}

	F.byWords = function(i) {
		var self = this
		i |= 0
		return function() {
			var s = this, r = s, a = arguments
			;(a[i]||"").replace(/[-\w]+/g, function(w){a[i]=w;r=self.apply(s, a)})
			return r
		}
	}

	F.byKeyVal = function() {
		var self = this
		return function(o) {
			var r, s = this, a = sl(arguments)
			if (typeof o == "object") for (r in o) {
				a[0] = r
				a[1] = o[r]
				r = self.apply(s, a)
			} else r = self.apply(s, a)
			return r
		}
	}

	// Run function once and return cached value or cached instance
	F.cache = function(instance, keyFn, cache) {
		var self = this, c = cache || {}, f = function() {
			var a = arguments
			, i = !!instance || this instanceof f
			, k = keyFn ? keyFn(a, self) : i + ":" + a.length + ":" + A.join.call(a)

			return k in c ? c[k] : (c[k] = i ? self.construct(a) : self.apply(this, a))
		}
		f.origin = self
		f.cached = c
		f.extend = function() {
			return self.extend.apply(self, arguments).cache(instance, keyFn, cache)
		}
		f[P] = self[P] // prototype for better access on extending 
		return f
	}

	F.extend = function() {
		var a, self = this, i = 0, f = function() {
			return self.apply(this, arguments)
		}
		f[P] = Object.create(self[P])
		f[P].constructor = f
		while (a = arguments[i++]) Object.merge(f[P], a)
		return f
	}

	/*

	F.chain = function(a) {
		return "a b->->b.call(this,a.apply(this,arguments))".fold(Array.isArray(a) ? a : sl(arguments), this)
	}

	F.compose = function() {
		var a = [this].concat(sl(arguments)), t = a.pop()
		return t.chain(a)
	}

	F.guard = function(test, or) {
		var self = this
		, f = test.fn()
		, o = (or||Nop).fn()

		return function() {
			return (f.apply(this, arguments) ? self : o).apply(this, arguments)
		}
	}
	*/

	// Time to live - Run *fun* if Function not called on time
	F.ttl = function(ms, fun) {
		var self = this, tick = setTimeout(function(){ms=0;fun&&fun()}, ms)
		return function() {
			clearTimeout(tick)
			ms && self.apply(null, arguments)
		}
	}

	// Run Function one time after last call
	F.once = function(ms) {
		var tick, args, self = this
		return function() {
			clearTimeout(tick)
			args = arguments
			tick = setTimeout(function(){self.apply(null, args)}, ms)
		}
	}

	// Maximum call rate for Function
	F.rate = function(ms, last_call) {
		var tick, args, self = this, next = 0
		return function() {
			var now = +new Date()
			clearTimeout(tick)
			if (now > next) {
				next = now + ms
				self.apply(null, arguments)
			} else if (last_call) {
				args = arguments
				tick = setTimeout(function(){self.apply(null, args)}, next-now)
			}
		}
	}


	/**
	 * Returns a function identical to this function except that
	 * it prints its arguments on entry and its return value on exit.
	 * This is useful for debugging function-level programs.
	 */

	/** debug.trace
	F.trace = function(n) {
		var self = this
		n = n || self
		return "console" in w ?
			function() {
			console.info('[', n, 'apply(', this!=w && this, ',', arguments, ')')
				var result = self.apply(this, arguments)
				console.info(']', n, ' -> ', result)
				return result
		} :
		self
	}
	//*/


	

	
	// Non-standard
	O.each = function(obj, fn, scope, key) {
		if (obj) for (key in obj) obj.hasOwnProperty(key) && fn.call(scope, obj[key], key, obj)
	}

	// Object.assign ( target, source ) in ECMAScript 6

	O.merge = function(target, source) {
		var k, o, i = 1
		while (o = arguments[i++]) for (k in o) if (o.hasOwnProperty(k)) target[k] = o[k]
		return target
	}

	O.deepMerge = function(target, source, path, changed, key, val) {
		path = path || ""
		changed = changed || []

		for (key in source) if (source.hasOwnProperty(key) && target[key] !== source[key]) {
			val = source[key]
			changed.push(path+key)
			if (val === null) delete target[key]
			else if (typeof val == "object" && typeof target[key] == "object")
				O.deepMerge(target[key], val, path+key+".", changed)
			else target[key] = val
		}
		return changed
	}

	O.zip = function(keys, vals) {
		return keys.fold(function(_, key, i) {
			_[key] = vals[i]
			return _
		}, {})
	}

	/*
	Array.flatten = function(arr) {
	for(var i=arr.length;i--;)
	0 in arr[i] && A.splice.apply(arr, [i, 1].concat(Array.flatten(arr[i])))
	return arr
	}
	flat([1,2,[3,4,[5,6]],7])
	*/
	// Non-standard
	// // IE < 9 bug: [1,2].splice(0).join("") == "" but should be "12"
	A.remove = function() {
		var self = this
		, l = self.length
		, o = sl(arguments)
		, last_id = -1

		while (l--) if (~o.indexOf(self[l])) self.splice(last_id = l, 1)
		return last_id
	}

	A.each = A.forEach
	A.fold = A.reduce
	A.foldr = A.reduceRight
	A.unique = A.filter.partial(function(s,i,a){return i == a.lastIndexOf(s)})

	!function(n) {
		F[n] = S[n] = function() {
			var a = arguments, l = a[0]
			a[0] = this.fn()
			return A[n].apply(l, a)
		}
	}.byWords()("every filter each map fold foldr some")


	F.fn = This

	S.fn = function() {
		return Fn(this)
	}

	/*
	* Copyright 2007 by Oliver Steele. MIT License
	* http://osteele.com/javascripts/functional
	* Modifyed by Lauri Rooden
	*/
	function Fn(expr) {
		if (Fn[expr]) return Fn[expr]
		var args = "_"
		, body = expr
		, arr = expr.split("->")

		while (arr.length > 1) {
			body = arr.pop()
			args = arr.pop().match(/\w+/g)||""
			arr.length && arr.push("(function("+args+"){return("+body+")})")
		}
		return Fn[expr] = new Function(args, "return(" + body + ")")
	}

	root.Fn = Fn

}(this)







/*
* @version  0.1.14
* @date     2014-01-02
* @author   Lauri Rooden - https://github.com/litejs/elements-lite
* @license  MIT License  - http://lauri.rooden.ee/mit-license.txt
*/


/* TODO: find ways for automated testing
* http://www.browserscope.org/user/tests/howto
*/


!function(root, doc, P) {
	var elCache = {}
	, fnCache = {}
	, proto = (root.HTMLElement || root.Element || El)[P]
	, el_re = /([.#:[])([-\w]+)(?:=([-\w]+)])?]?/g



	function El(name, args) {
		var el
		, pre = {}
		name = name.replace(el_re, function(_, o, s, val) {
			pre[
				o == "." ? (o = "class", (pre[o] && (s = pre[o]+" "+s)), o) :
				o == "#" ? "id" :
				s 
			] = val || s
			return ""
		}) || "div"

		el = (elCache[name] || (elCache[name] = doc.createElement(name))).cloneNode(true).set(pre)

		return fnCache[name] && fnCache[name](el, args) || el.set(args)
	}



	/* TODO: Extend El api
	* add El.siblings( [selector ] )
	* add El.children( [selector ] )
	* add El.invoke
	* https://github.com/WebReflection/dom4#dom4
	*/

	/*
	* How elements.js extends the DOM
	* -------------------------------
	*
	* All DOM extensions on the element are available by default.
	*
	* In browsers that does not support adding methods to prototype of native objects 
	* such as HTMLElement or Element, document.createElement will be overrided
	* to extend created elements. El.get() and element.find will extend
	* returned elements.
	*/



	/*
	* ### element.append( content, [ before ] ) -> element
	*
	* - **content** `element || Array || String || Number`
	* - **before** `optional`
	*     - true - Insert content to the beginning of element
	*     - element - Insert content before specified element
	*     - Number - Insert content before nth child
	*/

	proto.append = function(e, before) {
		var t = this
		if (e) {
			if (typeof e == "string" || typeof e == "number") e = El.text(e)
			else if ( !("nodeType" in e) && "length" in e ) {
				/*
				* document.createDocumentFragment is unsupported in IE5.5
				* f = "createDocumentFragment" in doc ? doc.createDocumentFragment() : El("div")
				*/
				
				for (
					var len = e.length
					, i = 0
					, f = doc.createDocumentFragment();
					i < len
					; ) t.append.call(f, e[i++]);
				e = f
			}

			if (e.nodeType) t.insertBefore(e,
				(before === true ? t.firstChild :
				typeof before == "number" ? t.childNodes[
					before < 0 ? t.childNodes.length - before - 2 : before
				] : before) || null
			)
			e.append_hook && e.append_hook()
			//"child_hook" in t && t.child_hook()
		}
		return t
	}

	proto.after = function(e, before) {
		e.parentNode.append(this, before ? e : e.nextSibling)
		return this
	}

	proto.to = function(e, before) {
		e.append(this, before)
		return this
	}

	proto.hasClass = function(name) {
		/*
		* http://jsperf.com/regexp-indexof-perf/32
		* return (" "+this.className+" ").indexOf(" "+name+" ") > -1
		*/
		return RegExp("\\b" + name + "\\b").test(this.className)
	}

	proto.addClass = function(name) {
		var t = this
		t.className += t.className == "" ? name : t.hasClass(name) ? "" : " " + name
		return t
	}

	proto.rmClass = function(name) {
		var t = this
		t.className = (" "+t.className+" ").replace(" "+name+" "," ").trim()
		return t
	}

	proto.toggleClass = function(name, s) {
		if (arguments.length == 1) s = !this.hasClass(name)
		this[ s ? "addClass" : "rmClass" ](name)
		return s
	}

	proto.empty = function() {
		var t = this, n
		while (n = t.firstChild) t.kill.call(n)
		return t
	}

	proto.kill = function() {
		var t = this
		t.parentNode && t.parentNode.removeChild(t)
		Event.removeAll && Event.removeAll(t)
		t.kill_hook && t.kill_hook()
		t.empty && t.empty()
		return t
	}

	proto.on = function(ev, fn) {
		Event.add(this, ev, fn)
		return this
	}

	proto.non = function(ev, fn) {
		Event.remove(this, ev, fn)
		return this
	}

	proto.set = function(args) {
		var t = this, k = typeof args, v
		if (args) {
			if (k == "string" || k == "number" || args.nodeType || "length" in args) t.append(args)
			else for (k in args) 
			/** hasOwnProperty
			if (args.hasOwnProperty(arg)) 
			//*/
			{
				v = args[k]
				// El uses class
				if (k == "class") t.addClass(v)
				else if (typeof v == "string") {
					/*
					* Note: IE5-7 doesn't set styles and removes events when you try to set them.
					*
					* in IE6, a label with a for attribute linked to a select list 
					* will cause a re-selection of the first option instead of just giving focus.
					* http://webbugtrack.blogspot.com/2007/09/bug-116-for-attribute-woes-in-ie6.html
					*/
					t.setAttribute(k, v)

					/*
					* there are bug in IE<9 where changed 'name' param not accepted on form submit
					* The JScript engine used in IE doesn't recognize vertical tabulation character
					* oldIE = "\v" == "v"
					* 
					* IE8 and below also support document.createElement('<P>')
					*
					* http://www.matts411.com/post/setting_the_name_attribute_in_ie_dom/
					* http://msdn.microsoft.com/en-us/library/ms536614(VS.85).aspx
					*/

					if (k == "name" && "\v" == "v") {
						t.mergeAttributes(doc.createElement("<INPUT name='" + k + "'/>"), false)
					}

				}
				else if (!v) t.removeAttribute(k)
				else t[k] = v
			}
		}
		return t
	}

	/*
	* Expose slow find for testing
	*
	* TODO: look another way
	* http://ajaxian.com/archives/creating-a-queryselector-for-ie-that-runs-at-native-speed
	*/

	proto._find = function(sel) {
		var el
		, i = 0
		, rules = ["_"]
		, tag = sel.replace(el_re, function(_, o, s, v) {
				rules.push(
					o == "." ? "(' '+_.className+' ').indexOf(' "+s+" ')>-1" :
					o == "#" ? "_.id=='"+s+"'" :
					"_.getAttribute('"+s+"')"+(v?"=='"+v+"'":"")
				)
				return ""
			}) || "*"
		, els = this.getElementsByTagName(tag)
		, fn = Function("_", "return " + rules.join("&&"))

		while (el = els[i++]) {
			if (fn(el)) {
				return el.to ? el : extend(el)
			}
		}
	}

	proto.find = doc.querySelector ?
		function(sel) {
			/*
			* Note: IE8 don't support :disabled
			*/
			return this.querySelector(sel)
		} : proto._find


	function extend(e, p, k) {
		if (e) {
			p = El[P]
			for (k in p) e[k] = p[k]
		}
		return e
	}



	if (proto === El[P]) {
		/*
		* IE 6-7
		*/

		var create = doc.createElement
		doc.createElement = function(name) {return extend(create(name))}

		extend(doc.body)

		/*
		* Remove background image flickers on hover in IE6
		*
		* You could also use CSS
		* html { filter: expression(document.execCommand("BackgroundImageCache", false, true)); }
		*/
		/*@cc_on try{document.execCommand('BackgroundImageCache',false,true)}catch(e){}@*/
	}

	El[P] = proto


	El.get = function(id) {
		if (typeof id == "string") id = doc.getElementById(id)
		return id && id.to ? id : extend(id)
	}

	El.cache = function(name, el, custom) {
		elCache[name] = typeof el == "string" ? El(el) : el
		if (custom) {
			fnCache[name] = custom
		}
	}
	El.cache.el = elCache
	El.cache.fn = fnCache
	El.text = function(str) {
		return doc.createTextNode(str)
	}
	root.El = El

}(this, document, "prototype")







/*
* @version    0.1.3
* @date       2014-01-20
* @stability  2 - Unstable
* @author     Lauri Rooden <lauri@rooden.ee>
* @license    MIT License
*/



!function(root) {
	// The addEventListener is supported in Internet Explorer from version 9.
	// https://developer.mozilla.org/en-US/docs/Web/Reference/Events/wheel

	var Event = root.Event || (root.Event={})
	, wheelDiff = 120
	, prefix = ""
	, addEv = "addEventListener"
	, remEv = "removeEventListener"
	, WHEEL_EVENT = 
		"onwheel" in root      ? "wheel" :      // Modern browsers support "wheel"
		"onmousewheel" in root ? "mousewheel" : // Webkit and IE support at least "mousewheel"
		"DOMMouseScroll"                        // let's assume that remaining browsers are older Firefox
	, Emitter = Event.Emitter = {
		on: function(ev, fn, scope) {
			var t = this
			, e = t._e || (t._e = {})
			;(e[ev] || (e[ev] = [])).push([fn, scope])
			return t
		},
		non: function(ev, fn) {
			var t = this
			if (ev) {
				if (t._e && t._e[ev]) {
					if (fn) for (var a = t._e[ev], l = a.length; l--;) if (a[l][0] == fn) a.splice(l, 1)
					else delete t._e[ev]
				}
			} else delete t._e
			return t
		},
		once: function(ev, fn, scope) {
			var t = this
			return t.on(ev, fn, scope).on(ev, t.non.bind(t, ev, fn))
		},
		emit: function(ev) {
			var t = this
			if (t._e && t._e[ev]) {
				for (var i=0, e=t._e[ev], a=e.slice.call(arguments, 1); ev=e[i++];) ev[0].apply(ev[1]||t, a)
			}
			return t
		}
	}

	// Alias
	Emitter.off = Emitter.non

	if (!root[addEv]) {
		prefix = "on"
		addEv  = "attachEvent"
		remEv  = "detachEvent"
	}


	function fixOn(el, type, fn) {
		//TODO: remove wraper from trusted wheel event
		//var fix = type == "wheel" && type !== WHEEL_EVENT ? 
		var fix = type == "wheel" ? 
			function(e) {
				if (!e) e = root.event
				var delta = (e.wheelDelta || -e.detail || -e.deltaY)/wheelDiff
				if (delta != 0) {
					if (delta < 1 && delta > -1) {
						var diff = (delta < 0 ? -1 : 1)/delta
						delta *= diff
						wheelDiff /= diff
					}
					//TODO: fix event
					// e.deltaY = 
					// e.deltaX = - 1/40 * e.wheelDeltaX|0
					// e.target = e.target || e.srcElement
					fn.call(el, e, delta)
				}
			} : 
			prefix ? function(){
				fn.call(el, root.event)
			} : fn

		if (fix != fn) fix.origin = fn
		Emitter.on.call(el, type, fix, el)
		return fix
	}

	function fixOff(el, type, fn) {
		if (fn && el._e && el._e[type]) {
			for (var _fn, arr = el._e[type], i = 0; _fn=arr[i];i++) { 
				_fn = _fn[0]
				if (_fn == fn || _fn.origin == fn) {
					arr.splice(i, 1)
					return _fn
				}
			}
		}
	}

	
	function raw(add, fix, el, ev, fn) {
		fn = fix(el, ev, fn)
		el[add](prefix + (ev == "wheel" ? WHEEL_EVENT : ev), fn, false)
		return Event
	}

	Event.add    = raw.bind(null, addEv, fixOn)
	Event.remove = raw.bind(null, remEv, fixOff)

	Event.stop = function(e) {
		e.stopPropagation && e.stopPropagation()
		e.preventDefault && e.preventDefault()
		e.cancelBubble = e.cancel = true
		return e.returnValue = false
	}

	Event.removeAll = function(el, ev, key, arr, i) {
		if (el._e) for (key in el._e) {
			if (!ev || key == ev) for (arr = el._e[key], i = arr.length; i--;) {
				Event.remove(el, key, arr[0])
			}
		}
	}

}(this)







/*
* @version  0.0.4
* @author   Lauri Rooden - https://github.com/litejs/browser-history-lite
* @license  MIT License  - http://lauri.rooden.ee/mit-license.txt
*/



!function(win, doc, his) {
	var cb, base, last_route, iframe, tick, last
	, loc = location
	, clean_route = /^[#\/\!]+|[\s\/]+$/g
	/*
	* The JScript engine used in IE doesn't recognize vertical tabulation character
	* http://webreflection.blogspot.com/2009/01/32-bytes-to-know-if-your-browser-is-ie.html
	* oldIE = "\v" == "v"
	*
	* The documentMode is an IE only property, supported in IE8+.
	*
	* Starting in Internet Explorer 9 standards mode, Internet Explorer 10 standards mode, 
	* and win8_appname_long apps, you cannot identify the browser as Internet Explorer 
	* by testing for the equivalence of the vertical tab (\v) and the "v". 
	* In earlier versions, the expression "\v" === "v" returns true. 
	* In Internet Explorer 9 standards mode, Internet Explorer 10 standards mode, 
	* and win8_appname_long apps, the expression returns false.
	*/
	, ie6_7 = !+"\v1" && (doc.documentMode||1) < 8

	function getUrl(_loc) {
		var url
		if (base) {
			url = loc.pathname.slice(base.length)
		} else {
			/*
			* bug in Firefox where location.hash is decoded
			* bug in Safari where location.pathname is decoded
			* 
			* var hash = loc.href.split('#')[1] || '';
			* https://bugs.webkit.org/show_bug.cgi?id=30225
			* https://github.com/documentcloud/backbone/pull/967
			*/
			url = (_loc || loc).href.split("#")[1] || ""
		}
		/*
		* Fix decode
		* http://unixpapa.com/js/querystring.html
		*/
		url = decodeURIComponent(url.replace(/\+/g, " "))
		return url.replace(clean_route, "")
	}

	function setUrl(url, replace) {
		url = encodeURIComponent(url).replace(/%20/g, "+").replace(/%2F/gi, "/")
		if (base) {
			his[replace ? "replaceState" : "pushState"](null, null, base + url)
		} else {
			loc[replace ? "replace" : "assign"]("#" + url)
			/*
			* Opening and closing the iframe tricks IE7 and earlier 
			* to push a history entry on hash-tag change.
			*/
			if (iframe && getUrl() !== getUrl(iframe.location) ) {
				iframe.location[replace ? "replace" : iframe.document.open().close(), "assign"]("#" + url)
			}
		}
		checkUrl()
	}

	function checkUrl() {
		if (last_route != (last_route = getUrl()) && cb) cb(last_route)
	}
	
	his.getUrl = getUrl
	his.setUrl = setUrl

	his.start = function(_cb, _base) {
		if (!cb) {
			if (_base && his.pushState) {
				base = _base
				/*
				* Chrome and Safari emit a popstate event on page load, Firefox doesn't.
				* Firing popstate after onload is as designed.
				*
				* See the discussion on https://bugs.webkit.org/show_bug.cgi?id=41372, 
				* https://code.google.com/p/chromium/issues/detail?id=63040
				* and the change to the HTML5 spec that was made: 
				* http://html5.org/tools/web-apps-tracker?from=5345&to=5346. 
				*/
				win.onpopstate = checkUrl
			} else if ("onhashchange" in win && !ie6_7) {
				/*
				* There are onhashchange in IE7 but its not get emitted
				*
				* Basic support: 
				* Chrome 5.0, Firefox (Gecko) 3.6 (1.9.2), IE 8.0, Opera 10.6, Safari 5.0
				*/
				win.onhashchange = checkUrl
			} else {
				if (ie6_7) {
					iframe = doc.body.appendChild(doc.createElement('<iframe class="hide" src="javascript:0" tabindex="-1" />')).contentWindow
				}
				last = getUrl()
				tick = setInterval(function(){
					var cur = getUrl()
					if (iframe && last === cur) cur = getUrl(iframe.location)
					if (last !== cur) {
						last = cur
						iframe ? setUrl(cur) : checkUrl()
					}
				}, 50)
			}
		}
		cb = _cb
		checkUrl()
	}
}(this, document, history)






/*
* GET	PUT	POST	DELETE
*
* The default sync handler maps CRUD to REST like so:
*
* create → POST   /collection
* read → GET   /collection[/id]
* update → PUT   /collection/id
* delete → DELETE   /collection/id
*/


!function(root) {
	var nameRe = /^:+|\?+$/g
	, optionalRe = /^$|\?$/

	function Router() {
		this.map = {}
	}

	function RouterAdd(path, opts) {
		var next
		, t = this
		, i = 0
		, junks = path.replace(/\/+$/, "").split("/")
		, obj = { 
			path: path,
			opts: typeof opts == "function" ? { "next": opts } : opts || {}
		}

		for (path = ""; ; ++i) {
			next = junks[i] || ""

			if (optionalRe.test(next)) {
				t.map[path] = t.map[path + "/"] = obj
			}

			if (!next) break

			if (next.match(nameRe)) {
				;(obj.names || (obj.names = {}))[i] = next.replace(nameRe, "");
				next = ":?"
			}
			path = path ? path + "/" + next : next
		}
	}

	function RouterRoute(path, opts) {
		opts = typeof opts == "function" ? { "next": opts } : opts || {}

		var map = this.map
		, route = map[path] || ""

		// Fast case
		if (route) {
			path = ""
		} else {
			var junks = path.split("/")
			, i = 0
			, len = junks.length


			

			for (; i < len; ++i) {
				next = route ? route + "/" + junks[i] : junks[i]
				if (!map[ next ]) {
					next = route ? route + "/:?" : ":?"
					if (map[next]) {
						;(opts.args || (opts.args = {}))[ map[next].names[i] ] = junks[i]
					} else break
				}
				route = next
			}
			route = map[route]
			path = junks.slice(i).join("/")
		}

		if (route.opts.before) {
			route.opts.before.call(route, opts)
		}

		;(opts.next || route.opts.next).call(route, opts)

		if (route.opts.after) {
			route.opts.after.call(route, opts)
		}
	}


	Router.prototype = 
		{ add: RouterAdd
		, route: RouterRoute
		}

	root.Router = Router
}(this)








/*
* @version  0.1.3
* @author   Lauri Rooden - https://github.com/litejs/view-lite
* @license  MIT License  - http://lauri.rooden.ee/mit-license.txt
*/






!function(root) {
	var views = {}
	, routes = new Router()

	function View(route, el, parent, selector) {
		var t = this
		if (views[route]) return views[route]
		if (!(t instanceof View)) return new View(route, el, parent, selector)
		views[route] = t
		routes.add(route)
		el && t.init.apply(t, arguments)
	}

	View.prototype = {
		init: function(route, el, parent, selector) {
			var t = this
			t.route   = route
			t.el      = el
			t.parent  = typeof parent == 'string' ? View(parent) : parent
			t.selector = selector
			if (View.active == route) t.show()
		},
		getEl: function() {
			var t = this
			, type = typeof t.el
			if (type == 'function') t.el = t.el()
			else if (type == 'string') t.el = El(t.el)
			return t.el
		},
		getContentEl: function() {
			var t = this
			
			return t.selector && t.getEl().find(t.selector) || t.getEl()
		},
		load: function(opts) {
			var t = this

		},
		show: function(opts){
			var t = this
			View.active = t.route
			if (t.el) {
				t.active && t.close()
				t.ping(opts)
				if (View.active == t.route) t.emit('show', opts)
			} else {
				t.load(opts)
			}
		},
		close: function(opts){
			var t = this
			, parent = t.parent
			
			t.active = false

			if (parent) {
				parent.getContentEl().removeChild(t.getEl())
				parent.child = null
			}
			t.emit('close', opts)
			t.child && t.child.close()
		},
		ping: function(opts){
			var t = this
			, parent = t.parent

			t.active = true

			if (parent) {
				if (parent.child != t) {
					parent.child && parent.child.close()
					parent.child = t
					parent.getContentEl().appendChild(t.getEl())
				}
				parent.ping(opts)
			}
			t.emit('ping', opts)
		}
	}

	Object.merge(View.prototype, Event.Emitter)

	function drawRoute(opts) {
		View(this.path).show(opts)
	}
	/*
	* TODO: 'default' is a reserved word, pick new one.
	* View.welcome
	* View.index
	* View.main
	* View.root
	*/

	routes.add('', { 'before': function(opts){
		opts.next = function() {
			routes.route(View['default'], drawRoute)
		}
	} })

	View.route = function(route) {
		routes.route(route || history.getUrl(), drawRoute)
	}

	root.View = View
}(this)

